#!/bin/sh
##########################################################################################
# Program Name : Gaf Replacer
# Version      : 
# Started Date : August 16, 2004 
# Description  :
#
#
##########################################################################################

############# Global Variable #############
set file_path ""
#Options
array set opt {
	mark_finds	0
	prompt		0
	case		0
	show_result	1
}
array set colours {
	found_fore		"white"
	found_back		"blue"
	replaced_fore 	"red"
	replaced_back 	"white"
	marked_fore		"blue"
	marked_back		"white"
}

array set program {
	name		"GaF Replacer"
	author		"Binny V Abraham"
	version		"1.00.A"
	email		"binnyva@hotmail.com"
	website		"http://www.geocites.com/binnyva"
}

############################ Funtions for Funtions ##########################
# Function : lineLength
# Return the list of all the line lengths in the given text. This is called from 
#		'replace' function. Needed to find the position of text in the text widget.
# Argument : content - The inputed text.
# Return   : lengths - The list of all line lengths
proc lineLength { content } {
	set lines [split $content "\n"]
	set total 0
	lappend lengths 0
	foreach line $lines {
		set total [expr $total + [string length $line] + 1]
		lappend lengths $total
	}
	return $lengths
}

# Function : findLine
# Find in which line is $i. The list containing all line length made by lineLength
#		functions.
# Argument : line_no - The position of the wanted char in the text
#			 line_lengths - The list containing the lengths of all lines made by lineLength
# Return   : cur_line - The line in which the char will be found
proc findLine { line_no line_lengths } {
	set cur_line -1
	for { set i 0 } { $i < [llength $line_lengths] } { incr i } {
		if {($line_no >= [lindex $line_lengths $i]) &&\
			($line_no < [lindex $line_lengths [expr $i + 1]])} {
			set cur_line $i
			break
		}
	}
	return $cur_line
}

# Function : promptReplace
# Displays a small window asking the user if he wishes to replace the currently selected
#		text. The user has to respond by pressing any key.
# Arguments: line - The line no of the selected text
#			 char - The charecter no of the selected text
# Return   : do_replace - The numarical value of user's choice
proc promptReplace { line char } {
	set do_replace 10
	set modify 0
	set wmx [winfo rootx .contents]
	set wmy [winfo rooty .contents]
	set wmx [expr $wmx + ( $char * 8 )  + 2]
	set wmy [expr $wmy + ( $line * 16 ) + 5]

	#Make sute that there is no widget called .prompt
	if {![winfo exists .prompt]} {
		#Feels extreamly wierd, don't it? I don't know why but varibles can't keep its
		#		values here. This was the only alternative.
		set file_path [.file_name get]
		.file_name delete 0 end
		
		#tk_dialog .prompt "Replace this occerence?" "Replace Text?" "" "0" "Yes" "No" "Cancel"
		toplevel .prompt
		label .prompt.lbl_question -text "Replace(Yes/No/All/Cancel)?"
		pack .prompt.lbl_question -in .prompt
		bind .prompt <y> {
			set modify 1 
			.file_name insert end "1"
			}
		bind .prompt <Y> {
			set modify 1
			.file_name insert end "1"
			}
		bind .prompt <n> {
			set modify 1
			.file_name insert end "2"
			}
		bind .prompt <N> {
			set modify 1
			.file_name insert end "2"
			}
		bind .prompt <a> {
			set modify 1
			.file_name insert end "3"
			}
		bind .prompt <A> {
			set modify 1
			.file_name insert end "3"
			}
		bind .prompt <c> {
			set modify 1
			.file_name insert end "4"
			}
		bind .prompt <C> {
			set modify 1
			.file_name insert end "4"
			}
		bind .prompt <Key-Escape> {
			set modify 1
			.file_name insert end "0"
			}
	
		catch {wm resizable .prompt 0 0} ;#Make it unresizable
		wm geometry .prompt [winfo reqwidth .prompt.lbl_question]x[winfo reqheight .prompt.lbl_question]+$wmx+$wmy ;# Put it in the correct place - just below the text to be replaced.
		raise .prompt
 		wm deiconify .prompt
	 	focus .prompt.lbl_question
		grab set .prompt

		vwait modify
		catch { destroy .prompt }
		set do_replace [.file_name get]
		.file_name delete 0 end 
		.file_name insert end "$file_path"		
	}
	return $do_replace
}

############################ Important Funtions ##########################
# Function : openFile
# Shows a file opening dialog and set the contents of the selected files in the
#		text area.
proc openFile { } {
	global file_path
	set types {
	{{Text Files} 		{.txt}	  }
	{{HTML Files}       {.html}   }
	{{HTML Files}       {.htm}    }
	{{JavaScript Files} {.js} 	  }
	{{C++ Files} 		{.cpp}	  }
	{{C Files} 			{.c}	  }
	{{Header Files} 	{.h}	  }
	{{Perl Files} 		{.pl}	  }
	{{Tcl/Tk Files} 	{.tcl}	  }
	{{Java Files} 		{.java}	  }
	{{All Files}        *         }
	}
	set result [tk_getOpenFile -filetypes $types] ;# Can add  -initialdir $user_folder
	if { $result != "" } {
		#Put the filename in the entry widget after clearing it
		.file_name delete 0 end
		.file_name insert end "$result"
		
		#But the contents in the contents region
		set in [open "$result" r]
		.contents delete 1.0 end
		.contents insert end [read $in]
		close $in
		
		.contents edit reset
		set file_path $result
	}
}

# Function : save
# Saves the changes to the current file.
proc save {} {
	global file_path
	#set file_path [.file_name get]
	set contents [.contents get 1.0 end]
	set out [open "$file_path" w]
	if { $out != 0 } {
		puts $out $contents
		flush $out
		stat "File saved to \"$file_path\""
	}
	close $out
}

# Function : replace
# The function that does the important job - replace the text
proc replace {} {
	global opt colours
	#Clear current tags
	.contents tag delete "link"
	.contents tag delete "marked"

	#Initializaitons
	set changed 0
	set found 0
	set re [.expression get]
	set replace [.replace get]
	regsub "\\\\n" $replace "\n" replace ;# Making '\n' chars valid
	set tags {}
	set i 0
	set seltxt ""
	set from_last_length 0
	set	to_last_length 0

	#Decides wheter the question should popup every time.
	set prompter 0
	if { $opt(prompt) } {
		set prompter 1
	}
	#Case-Sensitive replacing or not
	if { $opt(case) } {
		set caser {-line}
	} else {
		set caser {-nocase}
	}
	
	#Get the text and its info
	set content [.contents get 1.0 end]
	set line_lengths [lineLength $content]
	set length [string length $content]
	catch { set seltxt [selection get -displayof .contents STRING] }
	if { $seltxt != "" } {
		set selection_rages [.contents tag ranges "sel"] ;#Get the LINE.CHAR range of the selection
		# To get the starting line
		set selection_from [lindex $selection_rages 0]
		set selection_from [split $selection_from "."]
		set from_line [lindex $selection_from 0]
		# To get the ending line
		set selection_to [lindex $selection_rages 1]
		set selection_to [split $selection_to "."] 
		set to_line [lindex $selection_to 0]

		#Get the absolute charecter nos
		set i [lindex $line_lengths [expr $from_line - 1]]
		set length [lindex $line_lengths [expr $to_line - 1]]
		set i [expr $i + [lindex $selection_from 1]]
		set length [expr $length + [lindex $selection_to 1]]
	}

	while { $i < $length } {
		set flag [regexp $caser -line -lineanchor -linestop -start $i -indices "$re" $content index]
		if { $flag != 0 } {
			#Mark the replace
 			set from_to [split $index " "]
			set difference [expr [lindex $from_to 1] - [lindex $from_to 0]]
			set from [lindex $from_to 0]
			set to [lindex $from_to 1]
			#Find in which line '$from' and '$to' are located. This will be the line of the replace.
			set from_line [expr [findLine $from $line_lengths] + 1]
			set to_line [expr [findLine $to $line_lengths] + 1]
 			set from_last_length [lindex $line_lengths [expr $from_line - 1]]
 			set to_last_length [lindex $line_lengths [expr $to_line - 1]]
 			#Get thc char # with respect to the line.
			if { ($from_last_length != 0) && ($from_last_length != "")} {
				set from [expr $from - $from_last_length]
				set to [expr $to - $to_last_length + 1]
			} else {
				set to [expr [lindex $from_to 1] + 1]
			}

			#One by one replacing
			if { $prompter } {
				#Mark the text
				.contents tag add "found" "$from_line.$from" "$to_line.$to"
 				.contents tag configure "found" \
 					-foreground $colours(found_fore) -background $colours(found_back)

				#Open a window that ask a question
				set do_replace [promptReplace $to_line $from]
				#If we can replace...
				if { $do_replace == 1 } {
					set txt [.contents get "$from_line.$from" "$to_line.$to"] ;#Get the text
					regsub $caser -line -lineanchor -linestop "$re" $txt "$replace" txt ;#Find the replacement

					#Proof that the replacing was done.
					incr changed
					lappend tags "$from_line.$from $to_line.$to"

					.contents edit separator ;# Make it undo-able
					.contents delete "$from_line.$from" "$to_line.$to" ;#Delete the current text
 					.contents insert $from_line.$from "$txt" ;#And replace it.

				} elseif {($do_replace == 0)||($do_replace == 4)} {
					set length 0 ;#The Loop will end but won't continue.
					
				} elseif {$do_replace == 3||$do_replace == 10} {
					#The loop will never prompt again
					set prompter 0
					set content [.contents get 1.0 end]
					#Start the searching from the same char this loop began with
					set from_to [list [lindex $from_to 1] [lindex $from_to 0]]
				}

				#Remove the selection and markings
				.contents tag delete "found"
			} else {
				#All in one replacing
				regsub $caser -lineanchor -line -linestop -start $i "$re" $content "$replace" content
				incr changed
				lappend tags "$from_line.$from $to_line.$to"
			}
			incr found
			#Start the searching from the NEXT char
			set i [expr [lindex $from_to 1] + 1]
		} else {
			incr i
		}
	}

 	if { $changed } {
	 	if { !$prompter } {
		 	.contents edit separator ;# Make it undoable
			.contents delete 1.0 end
			.contents insert end "$content"			
		}
		#Mark the changes if so desired.
		if {$opt(mark_finds)} {
			foreach tag $tags {
				set from_to [split $tag " "]
				set from [lindex $from_to 0]
				set to [lindex $from_to 1]
				.contents tag add "link" "$from" "$to" 
			}		
			.contents tag configure "link" \
				-foreground $colours(replaced_fore) -background $colours(replaced_back)
		}
		stat "$changed changes made."
		if { $opt(show_result) } {
			if { $found == $changed } {
				set ch_msg "all the instances"
			} else {
				set ch_msg "only $changed"
			}
			tk_messageBox -message "$found instances of \"$re\" were found and $ch_msg were changed."
		}
 	} else {
	 	if { $opt(show_result) } {
		 	if { $found == 0 } {
			 	tk_messageBox -message "No instances of \"$re\" were found."
		 	} else {
			 	tk_messageBox -message "$found instances of \"$re\" were found but no changes were made."
		 	}
		}
	}
}

# Procedure : find
# Finds all instances of the specifed expression in the text and mark them all
proc find {} {
	global opt colours
	#Clear current tags
	.contents tag delete "link"
	.contents tag delete "marked"

	#Initializaitons
	set re [.expression get]
	set replace [.replace get]
	set tags {}
	set i 0
	set seltxt ""

	#Case-Sensitive replacing or not
	if { $opt(case) } {
		set caser {-line}
	} else {
		set caser {-nocase}
	}
	
	#Get the text and its info
	set content [.contents get 1.0 end]
	set line_lengths [lineLength $content]
	set length [string length $content]
	catch { set seltxt [selection get -displayof .contents STRING] }
	if { $seltxt != "" } {
		set selection_rages [.contents tag ranges "sel"] ;#Get the LINE.CHAR range of the selection
		# To get the starting line
		set selection_from [lindex $selection_rages 0]
		set selection_from [split $selection_from "."]
		set from_line [lindex $selection_from 0]
		# To get the ending line
		set selection_to [lindex $selection_rages 1]
		set selection_to [split $selection_to "."] 
		set to_line [lindex $selection_to 0]

		#Get the absolute charecter nos
		set i [lindex $line_lengths [expr $from_line - 1]]
		set length [lindex $line_lengths [expr $to_line - 1]]
		set i [expr $i + [lindex $selection_from 1]]
		set length [expr $length + [lindex $selection_to 1]]
	}

	while { $i < $length } {
		set flag [regexp $caser -line -lineanchor -linestop -start $i -indices "$re" $content index]
		if { $flag != 0 } {
			#Mark the replace
 			set from_to [split $index " "]
			set from [lindex $from_to 0]
			set to [lindex $from_to 1]
			#Find in which line '$from' and '$to' are located. This will be the line of the replace.
			set from_line [expr [findLine $from $line_lengths] + 1]
			set to_line [expr [findLine $to $line_lengths] + 1]
 			set from_last_length [lindex $line_lengths [expr $from_line - 1]]
 			set to_last_length [lindex $line_lengths [expr $to_line - 1]]
 			#Get thc char # with respect to the line.
			if { $from_last_length != 0 } {
				set from [expr $from - $from_last_length]
				set to [expr $to - $to_last_length + 1]
			} else {
				set to [expr [lindex $from_to 1] + 1]
			}

			#Mark the text
			.contents tag add "marked" "$from_line.$from" "$to_line.$to"
			.contents tag configure "marked" \
				-foreground $colours(marked_fore) -background $colours(marked_back)

			#Start the searching from the NEXT char
			set i [expr [lindex $from_to 1] + 1]
		} else {
			incr i
		}
	}
}

# Procedure : presetLoader
# Makes a menu from the data in file "presets.gaf"
# Arguments : m - ID of the menu
proc presetLoader { m } {
	#Make sure that no such menu exists
	if {![winfo exists "$m.cat_0"]} {
		#Open the file holding the presets, get the data, close it
		set pre_file [open "C:/Scripts/Under Constuction/Tcl/GaF Replacer/presets.gaf" r]
		seek $pre_file 0 start
		set presets [read $pre_file]
		close $pre_file
	
		#Initializations
		set menus ""
		set itmes ""
		set c "$m.cat_0"
		set cat_no 0
		set item_no 0
		
		#Get it as lines
		set lines [split $presets "\n"]
		foreach line $lines {
			if { $line != "" && ([string index $line 0]!="#") } {
				set flag [regexp "Catogory\:(.*)" $line full cat_name]
				if {$flag} {
					set c "$m.cat_$cat_no" ;# A unique id
					append menus "$m add cascade -label \"$cat_name\" -underline 0 -menu $c\n"
					append items "menu $c -tearoff 0\n"					
					incr cat_no
				} elseif { $line == "Separator" } {
					append items "$c add separator\n"
				} else {
					#If there is a currly bracket char in the text, change it to '\}'
					regsub "\}" $line "\\\}" line
					set parts [regexp "(.*) x@x (.*) x@x (.*) x@x (.*)" $line full title exp rep name]
					
					if { $name == "" } {
						set name "$title"
					}
					append items "$c add command -label \"$name\" -command { pre {$exp} {$rep} \"$title\" }\n"
				}
			}
		}
		#Execute the code we made
		eval "$menus\n$items"
	}
}

# Function : pre
# Restore the preset to its respective entrys when the menu is clicked.
# Arguments: expression - The expression to match the text.
#			 replace - The text with which 'expression' should be replaced.
#			 title - The name of the preset
proc pre { expression replace title } {
	#Delete the present expressions
	.expression delete 0 end
	.replace delete 0 end
	#Put in the new expressions
	.expression insert end "$expression"
	.replace insert end "$replace"
	stat "$title preset loaded."
}

# Function : peferences
# For Editing the preferences. Shows a window where the preferences can be set.
proc peferences { } {
	global opt colours
	#Make sute that there is no widget called .pref
	if {![winfo exists .preferences]} {
		set p [toplevel .preferences]
		set d [labelframe $p.lfm_display -relief groove -bd 2 -text "Display"]
		checkbutton $d.ckb_mark -text "Mark the Replaces" -variable opt(mark_finds)
		checkbutton $d.ckb_result -text "Show Results After Replace" -variable opt(show_result)

		set c [labelframe $p.lfm_colour -relief groove -bd 2 -text "Colours"]
		set f [label $c.lbl_found -text "Found Colours" -bg $colours(found_back) -fg $colours(found_fore)]
		button $c.but_found_fore -text "Font Colour" -command {
			set colours(found_fore) [setColour "Found" "Font" $colours(found_fore)]
		}
		button $c.but_found_back -text "Backgound Colour" -command {
			set colours(found_back) [setColour "Found" "Background" $colours(found_back)]
		}

		set r [label $c.lbl_replaced -text "Replaced Colours" -bg $colours(replaced_back) -fg $colours(replaced_fore)]
		button $c.but_replaced_fore -text "Font Colour" -command {
			set colours(replaced_fore) [setColour "Replaced" "Font" $colours(replaced_fore)]
		}
		button $c.but_replaced_back -text "Backgound Colour" -command {
			set colours(replaced_back) [setColour "Replaced" "Background" $colours(replaced_back)]
		}

		set m [label $c.lbl_marking -text "Marking Colours" -bg $colours(marked_back) -fg $colours(marked_fore)]
		button $c.but_marked_fore -text "Font Colour" -command {
			set colours(marked_fore) [setColour "Marking" "Font" $colours(marked_fore)]
		}
		button $c.but_marked_back -text "Backgound Colour" -command {
			set colours(marked_back) [setColour "Marking" "Background" $colours(marked_back)]
		}

		button $p.but_ok -text "    OK    " -command { catch { destroy .preferences } }

		#Geometry
		grid $d -in $p -row 1 -column 1 -sticky w
		grid $d.ckb_mark 	-in $d -row 1 -column 1 -sticky w
		grid $d.ckb_result	-in $d -row 2 -column 1 -sticky w
		grid $c -in $p -row 2 -column 1 -sticky w
		grid $f -in $c -row 1 -column 1 -sticky w
		grid $c.but_found_fore -in $c -row 1 -column 2 -padx 5
		grid $c.but_found_back -in $c -row 1 -column 3
		grid $r -in $c -row 2 -column 1 -sticky w
		grid $c.but_replaced_fore -in $c -row 2 -column 2 -padx 5
		grid $c.but_replaced_back -in $c -row 2 -column 3
		grid $m -in $c -row 3 -column 1 -sticky w
		grid $c.but_marked_fore -in $c -row 3 -column 2 -padx 5
		grid $c.but_marked_back -in $c -row 3 -column 3
		grid $p.but_ok -in $p -row 3 -column 1 -sticky w
		
		wm title $p "Preferences"
		focus $p.but_ok
	}
}

# Function : undoer
# Undos any changes made to the contests widget
proc undoer {} {
	.contents edit undo	
}

# Function : setColour
# Opens a dialog from which the user can select a colour and then return the colour
# Arguments: item - The item for which the colour should be set
#			 colour - The default colour
#			 bgfg - Background or Foreground
proc setColour { item bgfg colour } {
	set title "Select $bgfg Colour for $item text"
	set item ".preferences.lfm_colour.lbl_[string tolower $item]"
	set new_colour [tk_chooseColor -initialcolor $colour -title "$title"]
	
	wm deiconify .preferences
	focus .preferences.but_ok
	
	if {$new_colour != "" } {
		if { $bgfg == "Background" } {
			$item configure -background $new_colour
		} else {
			$item configure -foreground $new_colour
		}
		return $new_colour
	} else {
		return $colour
	}
}

# Procedure : makeMenu
# This function will make the menu bar at the top of the window. I thought
# 	that I will seperate it from the main GUI programming.
# Arguments : m - m is the id of the menu.
proc makeMenu { m } {
global file_path

menu $m
#The Main Buttons
$m add cascade -label "File" -underline 0 -menu [menu $m.file -tearoff 0]
$m add cascade -label "Edit" -underline 0 -menu [menu $m.edit -tearoff 0]
$m add cascade -label "Preferences" -underline 0 -menu [menu $m.pref -tearoff 0]
$m add cascade -label "Help" -menu [menu $m.help -tearoff 0]

## File Menu ##
set c $m.file
$c add command -label "New" -underline 0 -command { 
	.contents delete 1.0 end 
	set file_path ""
	.file_name delete 0 end 
	}
$c add command -label "Open" -underline 0 -command { openFile }
$c add command -label "Save" -underline 0 -command { save }
$c add command -label "Save As" -underline 5 -command { saveAs }
$c add separator
$c add command -label "Exit" -underline 1 -command exit

## Edit Menu ##
set c $m.edit
$c add command -label "Undo" -underline 0 -command { .contents edit undo }
$c add command -label "Redo" -underline 0 -command { .contents edit redo }
$c add separator
$c add command -label "Cut" -underline 2 -command {  }
$c add command -label "Copy" -underline 0 -command {  }
$c add command -label "Paste" -underline 0 -command {  }
$c add separator
$c add command -label "Select All" -underline 7 -command {  }
$c add cascade -label "Preset" -underline 3 -menu [menu $m.pre -tearoff 0]
presetLoader $m.pre
#$c add command -label "" -underline 0 -command {  }

## Preferences Menu ##
set c $m.pref
$c add command -label "Preferences" -underline 0 -command { peferences }
$c add command -label "Presets" -underline 3 -command {  }

## Help Menu ##
set c $m.help
$c add command -label "About" -underline 0 -command { aboutBox }
}

########################### GUI Functions ###########################

proc aboutBox { } {
	global program
	tk_messageBox -type ok -title "About GaF Replacer" -message \
"GaF Replacer V $program(version)

by Binny V Abraham

$program(email)
$program(website)"
}

# Function : Help
# Shows a Hint - copied from a program called 'Word Search 1.20'
proc help {w h} {
#Show the hint 1 second after the pointer rests on the same spot
bind $w <Any-Enter> "after 1000 [list help:show %W [list $h]]"
bind $w <Any-Leave> "destroy %W.balloon"
}
proc help:show {w arg} {
if {[eval winfo containing  [winfo pointerxy .]]!=$w} {return}
set top $w.balloon
catch {destroy $top}
toplevel $top -bd 0 -bg gray
wm overrideredirect $top 1
pack [message $top.txt -bg lemonchiffon1 -font ansi -width 170 \
	-text $arg -relief solid -aspect 10000 -bd 1 -padx 2 -pady 1]
set wmx [winfo rootx $w]
set wmy [expr [winfo rooty $w]+[winfo height $w]]
wm geometry $top [winfo reqwidth $top.txt]x[winfo reqheight $top.txt]+$wmx+$wmy
raise $top
}

#Show the specified text as the status for a second and then clear it
proc stat { msg } {
	.statusbar configure -text "$msg"
	after 2000 { .statusbar configure -text "Ready" }
}
#Update the status bar
proc status { msg } {
	if { $msg != "" } {
		.statusbar configure -text "$msg"
	} else {
		.statusbar configure -text "Ready"
	}
}

#Unwanted junk
proc tester { } {
	set seltxt [selection get -displayof .contents STRING]
	if { $seltxt != "" } {
		tk_messageBox -message "$seltxt"
	}
}

#################################### GUI Building ##################################
set heading "-Adobe-Helvetica-Bold-R-Normal--*-150-*-*-*-*-*-*"
set editing "-Adobe-Courier-*-R-Normal--*-130-*-*-*-*-*-*"

#ToolBar to the Left
frame .top
frame .toolbar
button .toolbar.open_file -text "Open" -command { openFile }
button .toolbar.save_file -text "Save" -command { save }
button .toolbar.undo -text "Undo" -command { catch { undoer } }
button .toolbar.pref -text "Preferences" -command { peferences }

#Preset Menu making...
menubutton .toolbar.presets -text "Presets" -menu .toolbar.presets.mnu \
	-border 2 -relief raised
help .toolbar.presets "Many often used replaces stored here for easy use."
set pre [menu .toolbar.presets.mnu -tearoff 0]
presetLoader $pre

##### Temporary #####
button .toolbar.temp -text "Temporary" -command "presetLoader $pre"
##### Temporary #####

#Makes the menu
makeMenu ".menubar"
. configure -menu .menubar

#File Contents -autoseparators true 
frame .frm_content
text .contents -font $editing -undo true \
	-exportselection true -wrap none \
	-yscrollcommand ".srly_content set" -xscrollcommand ".srlx_content set"
scrollbar .srly_content -command ".contents yview" -orient v
scrollbar .srlx_content -command ".contents xview" -orient h

#Replacing opitons - Bottom
frame .down
frame .frm_replaces -relief groove -bd 2
	label .frm_replaces.lab_expr -text "Expression" -width 11
	entry .expression
	set ins [menu .frm_replaces.insert_exp_mnu -tearoff 0]
	menubutton .frm_replaces.insert_exp -text "Insert" -menu $ins -border 2 -relief raised
		$ins add command -label "Any Charecter" -underline 0 -command { .expression insert insert "." }
		$ins add command -label "Begining of Line" -underline 0 -command { .expression insert insert "^" }
		$ins add command -label "End of line" -underline 0 -command { .expression insert insert "$" }
		$ins add command -label "New Line Charecter" -underline 0 -command { .expression insert insert "\\n" }
		$ins add separator
		$ins add command -label "0 or More Matches" -underline 0 -command { .expression insert insert "*" }
		$ins add command -label "1 or More Matches" -underline 0 -command { .expression insert insert "+" }
		$ins add command -label "0 or 1 Match" -underline 7 -command { .expression insert insert "?" }
		$ins add command -label "Or" -underline 0 -command { .expression insert insert "|" }
		$ins add command -label "Charecter in Range" -underline 13 -command { .expression insert insert "\[\]" }
		$ins add command -label "Charecter Not in Range" -underline 10 -command { .expression insert insert "\[^\]" }
		$ins add separator
		$ins add command -label "Alphabetic Charecter" -underline 12 -command { .expression insert insert "\[a-zA-Z\]" }
		$ins add command -label "Alphanumeric Charecter" -underline 7 -command { .expression insert insert "\\w" }
		$ins add command -label "Decimal Digit" -underline 0 -command { .expression insert insert "\\d" }
		$ins add separator
		$ins add command -label "Tagged Expression" -underline 0 -command { .expression insert insert "()" }

	label .frm_replaces.lab_replace -text "Replace With" -width 11
	entry .replace
	set ins [menu .frm_replaces.insert_rep_mnu -tearoff 0]
	menubutton .frm_replaces.insert_rep -text "Insert" -menu $ins -border 2 -relief raised
		$ins add command -label "Entire Matched Expression" -underline 0 -command { .replace insert insert "\\0" }
		$ins add separator
		$ins add command -label "Tagged Expression 1" -underline 18 -command { .replace insert insert "\\1" }
		$ins add command -label "Tagged Expression 2" -underline 18 -command { .replace insert insert "\\2" }
		$ins add command -label "Tagged Expression 3" -underline 18 -command { .replace insert insert "\\3" }
		$ins add command -label "Tagged Expression 4" -underline 18 -command { .replace insert insert "\\4" }
		$ins add command -label "Tagged Expression 5" -underline 18 -command { .replace insert insert "\\5" }
		$ins add command -label "Tagged Expression 6" -underline 18 -command { .replace insert insert "\\6" }
		$ins add command -label "Tagged Expression 7" -underline 18 -command { .replace insert insert "\\7" }
		$ins add command -label "Tagged Expression 8" -underline 18 -command { .replace insert insert "\\8" }
		$ins add command -label "Tagged Expression 9" -underline 18 -command { .replace insert insert "\\9" }

frame .frm_opts -relief groove -bd 2
checkbutton .frm_opts.ckb_prompt -text "Prompt on Replace" -variable opt(prompt)
checkbutton .frm_opts.ckb_case -text "Case Sensitive" -variable opt(case)

button .but_replace -text "Start Replacing" -command { replace }
help .but_replace "Replaces all instences of the specifed text in the file."
button .but_find -text "Find Expression" -command { find }
help .but_replace "Find and mark all instences of the specifed text in the file."

#Status Bar
label .statusbar -text "Ready" -justify left
entry .file_name -width 50 ;# -state readonly

#Geometry Management
#Toolbar 
grid .top -in . -row 1 -column 1 -sticky ew -columnspan 2
grid .toolbar -in .top -row 1 -column 1 -sticky n
grid .toolbar.open_file -in .toolbar -row 1 -column 1 -sticky w -pady 5 -padx 5
grid .toolbar.save_file -in .toolbar -row 2 -column 1 -sticky w -pady 5 -padx 5
grid .toolbar.undo -in .toolbar -row 3 -column 1 -sticky w -pady 5 -padx 5
grid .toolbar.pref -in .toolbar -row 4 -column 1 -sticky w -pady 5 -padx 5
grid .toolbar.presets -in .toolbar -row 5 -column 1 -sticky w -pady 5 -padx 5

grid .toolbar.temp -in .toolbar -row 6 -column 1 -sticky w -pady 25 -padx 5

#File Contents
grid .frm_content -in .top -row 1 -column 2
grid .contents -in .frm_content -row 1 -column 1 -sticky nsew
grid .srly_content -in .frm_content -row 1 -column 2 -sticky ns
grid .srlx_content -in .frm_content -row 2 -column 1 -sticky we

grid .down -in . -row 3 -column 1 -columnspan 2 -sticky w
#Replace Texts
grid .frm_replaces -in .down -row 1 -column 1 -padx 5 -pady 5 -ipadx 2 -ipady 2
grid .frm_replaces.lab_expr 	-in .frm_replaces -row 1 -column 1
grid .expression 				-in .frm_replaces -row 1 -column 2
grid .frm_replaces.insert_exp   -in .frm_replaces -row 1 -column 3 -padx 5
grid .frm_replaces.lab_replace 	-in .frm_replaces -row 2 -column 1
grid .replace 					-in .frm_replaces -row 2 -column 2
grid .frm_replaces.insert_rep   -in .frm_replaces -row 2 -column 3 -padx 5
#Replace Options
grid .frm_opts 	-in .down -row 1 -column 2 -sticky w -padx 5
grid .frm_opts.ckb_prompt -in .frm_opts -row 1 -column 1 -sticky w
grid .frm_opts.ckb_case   -in .frm_opts -row 2 -column 1 -sticky w

grid .but_replace -in .down -row 2 -column 1 -sticky w
grid .but_find -in .down -row 2 -column 2 -sticky w

#Status Bar
grid .statusbar -in . -row 4 -column 1 -sticky w 
grid .file_name -in . -row 4 -column 2 -sticky e


#This happens at the beginning - not really neesary.
proc init {} {
	global file_path
# 	set file_path "D:/Information/code/information.html"
	set file_path "C:/Scripts/Under Constuction/Tcl/GaF Replacer/Example.txt"
	.file_name insert end $file_path
	#But the contents in the contents region
	set in [open "$file_path" r]
	.contents insert end [read $in]
	close $in
	.contents edit reset
	.expression insert end "binny"
	.replace insert end "bijoy\\n"
	#.frm_opts.ckb_prompt select
}
init

wm title . "GaF Replacer"
bind . <Key-Escape> {exit}
focus .


#################################### To Do ##################################
# Menu
#	Implement all menu funcitons
# Correct Undo-ing : I think that the problem is a Tk bug.
# Multiple File Replacing
# If the replacing text has '\n' lots of problems will come